/*
    ================================================================================
    N-PUZZLE BOARD GAME
    Copyright (c) Jarmo Pietiläinen (a.k.a. z0b) 2008-2013
    http://z0b.kapsi.fi
    --------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License version 2.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51
    Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    ================================================================================
*/

// ----- NOTE -----
// The code in this file is based to the sample console code published by
// Alan Baylis (http://www.alsprogrammingresource.com/), who might have
// got the original code from http://dslweb.nwnexus.com/~ast/dload/guicon.htm
// (compare the codes - you'll see that they are nearly identical)
// ----------------

#undef __STRICT_ANSI__
#define UNICODE
#define _WIN32_WINNT 0x0500
#include <windows.h>

#include <cstdio>
#include <fcntl.h>

#include "debugconsole.h"

using namespace std;

#ifndef NO_DEBUG_CONSOLE
// filters Ctrl+C and Ctrl+Break, probably taken from
// http://www.codeproject.com/win32/console_event_handling.asp
static BOOL WINAPI customHandler(DWORD type)
{
    switch (type) {
        case CTRL_C_EVENT:
        case CTRL_BREAK_EVENT:
            return TRUE;
    }

    return FALSE;
}
#endif

namespace console
{
#ifndef NO_DEBUG_CONSOLE
    // console handles
    static HANDLE consoleHandle = INVALID_HANDLE_VALUE;
    static FILE *file[3] = { NULL };
#endif

    // opens the console window
    bool open(const wchar_t *title)
    {
#ifndef NO_DEBUG_CONSOLE
        if (consoleHandle != INVALID_HANDLE_VALUE)
            return false;

        CONSOLE_SCREEN_BUFFER_INFO info;

        // allocate a console for this app
        AllocConsole();

        // Set the screen buffer to be big enough to let us scroll text.
        // Windows obeys this size only on the first time, if user changes
        // console size, these are ignored.
        GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &info);
        info.dwSize.X = 80;
        info.dwSize.Y = 25;
        SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), info.dwSize);

        // set title
        SetConsoleTitle(title);

        // disable various system menu items and the close button
        HWND hc=GetConsoleWindow();
        HMENU m=GetSystemMenu(hc, 0);

        if (m) {
            DeleteMenu(m, SC_CLOSE, MF_BYCOMMAND);
            DeleteMenu(m, SC_SIZE, MF_BYCOMMAND);
            DrawMenuBar(hc);
        }

        // disable Ctrl+C and Ctrl+Break
        SetConsoleCtrlHandler(&customHandler, TRUE);

        int handle = 0;

        // redirect unbuffered STDERR to the console
        consoleHandle = GetStdHandle(STD_ERROR_HANDLE);
        handle = _open_osfhandle((long)consoleHandle, _O_TEXT);
        file[0] = _fdopen(handle, "w");
        *stderr = *file[0];
        setvbuf(stderr, NULL, _IONBF, 0);

        // redirect unbuffered STDIN to the console
        consoleHandle = GetStdHandle(STD_INPUT_HANDLE);
        handle = _open_osfhandle((long)consoleHandle, _O_TEXT);
        file[1] = _fdopen(handle, "r");
        *stdin = *file[1];
        setvbuf(stdin, NULL, _IONBF, 0);

        // redirect unbuffered STDOUT to the console
        consoleHandle = GetStdHandle(STD_OUTPUT_HANDLE);
        handle = _open_osfhandle((long)consoleHandle, _O_TEXT);
        file[2] = _fdopen(handle, "w");
        *stdout = *file[2];
        setvbuf(stdout, NULL, _IONBF, 0);
#endif

        return true;
    }

    // close the console
    void close()
    {
#ifndef NO_DEBUG_CONSOLE
        if (consoleHandle == INVALID_HANDLE_VALUE)
            return;

        FreeConsole();
        fclose(file[0]);
        fclose(file[1]);
        fclose(file[2]);
        consoleHandle = INVALID_HANDLE_VALUE;
#endif
    }

    // returns true if console is open, false if not
    bool isOpen()
    {
#ifndef NO_DEBUG_CONSOLE
        return consoleHandle == INVALID_HANDLE_VALUE ? false : true;
#else
        return false;
#endif
    }

    // clear the console
    void clear()
    {
#ifndef NO_DEBUG_CONSOLE
        if (consoleHandle == INVALID_HANDLE_VALUE)
            return;

        CONSOLE_SCREEN_BUFFER_INFO info;
        DWORD out;

        GetConsoleScreenBufferInfo(consoleHandle, &info);
        FillConsoleOutputCharacter(consoleHandle, L' ', info.dwSize.X * info.dwSize.Y, (COORD){ 0, 0 }, &out);
        SetConsoleCursorPosition(consoleHandle, (COORD){ 0, 0 });
#endif
    }
}
