// Simple pretty printer example. Okay, it's not that "pretty"...
// Public domain.

#include <stdio.h>
#include <ncfg.h>

#ifdef NCFG_WINDOWS

#include <windows.h>

static wchar_t *toUCS2(const char *utf8)
{
    if (!utf8)
        return NULL;

    const int len = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS,
                                        utf8, strlen(utf8), NULL, 0);

    if (len == 0)
        return NULL;

    wchar_t *ucs2 = (wchar_t *)malloc(sizeof(wchar_t) * (len + 1));

    if (!ucs2)
        return NULL;

    memset(ucs2, 0, sizeof(wchar_t) * (len + 1));

    if (!MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, utf8,
                             strlen(utf8), ucs2, len)) {
        free(ucs2);
        return NULL;
    }

    return ucs2;
}

#endif

#define INDENT_SIZE     4

void indent(const size_t level)
{
    // inefficient, I know
    for (size_t n = 0; n < INDENT_SIZE * level; n++)
        printf(" ");
}

void pretty_print(const struct cfg_section_t *s, const size_t level)
{
    if (!s)
        return;

    for (const struct cfg_item_t *item = s->head; item; item = item->next) {
        indent(level);
        printf("%s", item->key);

        for (size_t n = 0; n < item->size; n++)
            printf(" %s", item->value[n]);

        printf("\n");

        if (item->child_section) {
            indent(level);
            printf("{\n");
            pretty_print(item->child_section, level + 1);
            indent(level);
            printf("}\n");
        }
    }
}

int main(int argc, char *argv[])
{
    struct cfg_t *cfg = NULL;

    if (argc != 2) {
        printf("Usage: loadtest <file name>\n");
        return 1;
    }

    cfg_new(&cfg);

#ifdef NCFG_WINDOWS
    wchar_t *wide_name = toUCS2(argv[1]);

    if (cfg_load(cfg, wide_name) != CFG_OK) {
#else
    if (cfg_load(cfg, argv[1]) != CFG_OK) {
#endif
        printf("%s (%d)\nLine: %u\nColumn: %u\n",
                cfg_error_code_to_string(cfg->error_code), cfg->error_code,
                cfg->error_row, cfg->error_col);

        cfg_delete(&cfg);
        return 1;
    }

#ifdef NCFG_WINDOWS
    free(wide_name);
#endif

    pretty_print(cfg->root, 0);

    cfg_delete(&cfg);
    return 0;
}
