/*
    ================================================================================
    N-PUZZLE BOARD GAME
    Copyright (c) Jarmo Pietiläinen (a.k.a. z0b) 2008-2013
    http://z0b.kapsi.fi
    --------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License version 2.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51
    Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    ================================================================================
*/

// The actual puzzle class. 100% platform independent code.

#ifndef _NPUZZLE_PUZZLE_H
#define _NPUZZLE_PUZZLE_H

#include <vector>

#include "globaldefs.h"

class npuzzle {
private:
    uint32_t width, height;
    std::vector<uint32_t> board;

    void swapBlocks(const uint32_t a, const uint32_t b);

public:
    enum { EMPTY = 0, INVALID = 0xFFFFFFFF };

    npuzzle();
    ~npuzzle();

    // returns true if we have a valid board
    bool isValid() const;

    // smallest valid board size is 3x3, no upper limit
    bool create(const uint32_t width = 4, const uint32_t height = 4);

    // resets board to "solved" state
    void reset();

    // if iteration count is zero, board size is used to
    // determine "suitable" number of iterations
    void scramble(const uint32_t iterations = 128);

    // returns true if the board is solved
    bool isSolved() const;

    // returns the tile index at (x, y) or INVALID
    // if coordinates are not valid
    uint32_t tileAt(const uint32_t x, const uint32_t y) const;

    // returns the (X, Y) coordinates of the empty tile
    void getEmptyBlock(uint32_t &x, uint32_t &y) const;

    // move rows/columns
    bool move(const uint32_t x, const uint32_t y);
};

#endif
