/*
    ================================================================================
    N-PUZZLE BOARD GAME
    Copyright (c) Jarmo Pietiläinen (a.k.a. z0b) 2008-2013
    http://z0b.kapsi.fi
    --------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License version 2.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51
    Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    ================================================================================
*/

#undef __STRICT_ANSI__
#define UNICODE
#define _UNICODE
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shlwapi.h>

#include "util.h"

using namespace std;

namespace util
{
    // returns true if file exists
    bool fileExists(const wstring &name)
    {
        return !!PathFileExists(name.c_str());
    }
}

namespace ui
{
    // printf()-style SetWindowText()
    void setWindowText(HWND wnd, const wchar_t *fmt, ...)
    {
        if (!fmt || !wnd)
            return;

        wchar_t *buf = NULL;
        uint32_t size = 128U;
        va_list ap;

        va_start(ap, fmt);

        while (true) {
            buf = (wchar_t *)malloc(sizeof(wchar_t) * (size + 1U));

            if (!buf)
                return;

            memset(buf, 0, sizeof(wchar_t) * (size + 1U));

            if (vsnwprintf(buf, size - 1U, fmt, ap) == -1)
                size *= 2U;
            else break;

            if (size > 8192U) {
                free(buf);
                return;
            }
        }

        SetWindowText(wnd, buf);
        free(buf);
    }

    // retrieves an integer from an edit box
    void getIntFromEditBox(HWND wnd, int &i, const int def)
    {
        if (!wnd) {
            i = def;
            return;
        }

        BOOL translated;
        const UINT v = GetDlgItemInt(GetParent(wnd), GetDlgCtrlID(wnd), &translated, TRUE);

        if (!translated)
            i = def;
        else i = (int)v;
    }

    void getUIntFromEditBox(HWND wnd, uint32_t &i, const uint32_t def)
    {
        if (!wnd) {
            i = def;
            return;
        }

        BOOL translated;
        const UINT v = GetDlgItemInt(GetParent(wnd), GetDlgCtrlID(wnd), &translated, TRUE);

        if (!translated)
            i = def;
        else i = (uint32_t)v;
    }

    // retrieves a float number from an edit box
    void getFloatFromEditBox(HWND wnd, float &f, const float def)
    {
        if (!wnd) {
            f = def;
            return;
        }

        const int len = GetWindowTextLength(wnd);

        if (len == 0) {
            f = def;
            return;
        }

        wchar_t text[len + 2];

        GetWindowText(wnd, text, len + 1);
        f = _wtof(text);
    }
}

// Unicode conversion stuff
namespace unicode
{
    // UCS-2 -> UTF-8
    string toUTF8(const wstring &ucs2)
    {
        const int len = WideCharToMultiByte(CP_UTF8, 0, ucs2.c_str(),
                                            ucs2.length(), NULL, 0, NULL, NULL);

        if (len == 0)
            return string("");

        char *utf8 = (char *)malloc(sizeof(char) * (len + 1));

        if (!utf8)
            return string("");

        memset(utf8, 0, sizeof(char) * (len + 1));

        if (!WideCharToMultiByte(CP_UTF8, 0, ucs2.c_str(), ucs2.length(),
                                 utf8, len, NULL, NULL)) {
            free(utf8);
            return string("");
        }

        const string str(utf8);

        free(utf8);
        return str;
    }

    // UTF-8 -> UCS-2
    wstring toUCS2(const string &utf8)
    {
        const int len = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS,
                                            utf8.c_str(), utf8.length(), NULL, 0);

        if (len == 0)
            return wstring(L"");

        wchar_t *ucs2 = (wchar_t *)malloc(sizeof(wchar_t) * (len + 1));

        if (!ucs2)
            return wstring(L"");

        memset(ucs2, 0, sizeof(wchar_t) * (len + 1));

        if (!MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, utf8.c_str(),
                                 utf8.length(), ucs2, len)) {
            free(ucs2);
            return wstring(L"");
        }

        const wstring str(ucs2);

        free(ucs2);
        return str;
    }
}
