/*
    ================================================================================
    N-PUZZLE BOARD GAME
    Copyright (c) Jarmo Pietiläinen (a.k.a. z0b) 2008-2013
    http://z0b.kapsi.fi
    --------------------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License version 2.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc., 51
    Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    ================================================================================
*/

#ifndef _NPUZZLE_UTIL_H
#define _NPUZZLE_UTIL_H

#include <windows.h>
#include <cstdarg>
#include <string>

// utility functions
namespace util
{
    bool fileExists(const std::wstring &name);

    // swaps two variables
    template<class T> inline void swap(T &a, T &b)
    {
        const T t = a;

        a=b;
        b=t;
    }

    // clamps value to a range
    template<class T> inline T clamp(const T value, const T lower, const T upper)
    {
        return std::min(std::max(value, lower), upper);
    }
}

// UI helper stuff
namespace ui
{
    void setWindowText(HWND w, const wchar_t *s, ...);
    void getIntFromEditBox(HWND w, int &i, const int def = 0);
    void getUIntFromEditBox(HWND w, uint32_t &i, const uint32_t def = 0U);
    void getFloatFromEditBox(HWND w, float &f, const float def = 0.0f);
}

// Unicode glue
namespace unicode
{
    std::string toUTF8(const std::wstring &ucs2);
    std::wstring toUCS2(const std::string &utf8);
}

#endif
